package org.ntnu.masteroppg.biotracer;

import java.io.IOException;

import org.apache.lucene.index.*;
import org.apache.lucene.search.Explanation;
import org.apache.lucene.search.HitCollector;
import org.apache.lucene.search.Scorer;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.search.Weight;
import org.apache.lucene.document.Document;


/** Expert: A <code>Scorer</code> for documents matching a <code>Term</code>.
 */
final class LemurTFTermScorer extends Scorer {
  private Weight weight;
  private TermDocs termDocs;
  private IndexReader indexReader;
  private float weightValue;
  private int doc;
  private static final int SCORE_CACHE_SIZE = 32;
  private final int[] docs = new int[SCORE_CACHE_SIZE];         // buffered doc numbers
  private final int[] freqs = new int[SCORE_CACHE_SIZE];        // buffered term freqs
  private final int[] docLens = new int[SCORE_CACHE_SIZE];        // buffered term freqs
  private int pointer;
  private int pointerMax;
  
  /* BM25 Parameters */
  private final float DEFAULT_k1 = 1.2f;
  private final float DEFAULT_b = 0.75f;
  private float k_1=DEFAULT_k1, b=DEFAULT_b;
  
  private final boolean USE_TERRIER=true;
  
  private double queryKeyTf; // Query term frequency

  static float avgDocLen = 194.080f;

  
  private float[] scoreCache = new float[SCORE_CACHE_SIZE];

  /** Construct a <code>TermScorer</code>.
   * @param weight The weight of the <code>Term</code> in the query.
   * @param td An iterator over the documents matching the <code>Term</code>.
   * @param similarity The </code>Similarity</code> implementation to be used for score computations.
   * @param norms The field norms of the document fields for the <code>Term</code>.
   */
  LemurTFTermScorer(Weight weight, TermDocs td, Similarity similarity,
             IndexReader reader) {
    super(similarity);
    this.weight = weight;
    this.termDocs = td;
    this.indexReader = reader;
    this.weightValue = weight.getValue(); //idf*keyFactor() see CustomSimilarity

  }

  	
  
	public void setParams(float k1, float the_b) {
		k_1 = k1;
		b = the_b;
	}
	
	public void setParams(float the_b) {
		b = the_b;
	}
	
	public void setAvgDocumentLength(int avglen) {
		avgDocLen = avglen;
	}
	
  
  public void score(HitCollector hc) throws IOException {
    next();
    score(hc, Integer.MAX_VALUE);
  }

  private float term_score(float docLen, float tf) {
	  float Robertson_tf = k_1*tf/(tf+k_1*(1-b+b*docLen/avgDocLen));
	  
	  return (weightValue*Robertson_tf);
  }
  
  protected boolean score(HitCollector c, int end) throws IOException {
    Similarity similarity = getSimilarity();      // cache sim in local
    
    while (doc < end) {                           // for docs in window
      int f = freqs[pointer];
      Document document = indexReader.document(doc);
      float docLen = Float.valueOf(document.get("DOCLEN"));
      float score = term_score(docLen, f);

      //score *= normDecoder[norms[doc] & 0xFF];    // normalize for field

      c.collect(doc, score);                      // collect score

      if (++pointer >= pointerMax) {
        pointerMax = termDocs.read(docs, freqs);  // refill buffers
        if (pointerMax != 0) {
          pointer = 0;
        } else {
          termDocs.close();                       // close stream
          doc = Integer.MAX_VALUE;                // set to sentinel value
          return false;
        }
      } 
      doc = docs[pointer];
    }
    return true;
  }

  /** Returns the current document number matching the query.
   * Initially invalid, until {@link #next()} is called the first time.
   */
  public int doc() { return doc; }

  /** Advances to the next document matching the query.
   * <br>The iterator over the matching documents is buffered using
   * {@link TermDocs#read(int[],int[])}.
   * @return true iff there is another document matching the query.
   */
  public boolean next() throws IOException {
    pointer++;
    if (pointer >= pointerMax) {
      pointerMax = termDocs.read(docs, freqs);    // refill buffer
      if (pointerMax != 0) {
        pointer = 0;
      } else {
        termDocs.close();                         // close stream
        doc = Integer.MAX_VALUE;                  // set to sentinel value
        return false;
      }
    } 
    doc = docs[pointer];
    return true;
  }

  public float score() {
	  float score=0;
	  try {
	    int f = freqs[pointer];
	    Document document = indexReader.document(doc);
	    float docLen = Float.valueOf(document.get("DOCLEN"));
	    score = term_score(docLen, f);
	
	    
	  } catch (Exception e) {
		  e.printStackTrace();
	  }
	  return score; // normalize for field
  }

  /** Skips to the first match beyond the current whose document number is
   * greater than or equal to a given target. 
   * <br>The implementation uses {@link TermDocs#skipTo(int)}.
   * @param target The target document number.
   * @return true iff there is such a match.
   */
  public boolean skipTo(int target) throws IOException {
    // first scan in cache
    for (pointer++; pointer < pointerMax; pointer++) {
      if (docs[pointer] >= target) {
        doc = docs[pointer];
        return true;
      }
    }

    // not found in cache, seek underlying stream
    boolean result = termDocs.skipTo(target);
    if (result) {
      pointerMax = 1;
      pointer = 0;
      docs[pointer] = doc = termDocs.doc();
      freqs[pointer] = termDocs.freq();
    } else {
      doc = Integer.MAX_VALUE;
    }
    return result;
  }

  /** Returns an explanation of the score for a document.
   * <br>When this method is used, the {@link #next()} method
   * and the {@link #score(HitCollector)} method should not be used.
   * @param doc The document number for the explanation.
   */
  public Explanation explain(int doc) throws IOException {
    CustomTermQuery query = (CustomTermQuery)weight.getQuery();
    Explanation tfExplanation = new Explanation();
    int tf = 0;
    while (pointer < pointerMax) {
      if (docs[pointer] == doc)
        tf = freqs[pointer];
      pointer++;
    }
    if (tf == 0) {
        if (termDocs.skipTo(doc))
        {
            if (termDocs.doc() == doc)
            {
                tf = termDocs.freq();
            }
        }
    }
    termDocs.close();
    tfExplanation.setValue(getSimilarity().tf(tf));
    tfExplanation.setDescription("tf(termFreq("+query.getTerm()+")="+tf+")");
    
    return tfExplanation;
  }

  /** Returns a string representation of this <code>TermScorer</code>. */
  public String toString() { return "scorer(" + weight + ")"; }
}