package org.ntnu.masteroppg.eval;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.apache.lucene.queryParser.ParseException;
import org.ntnu.masteroppg.Config;
import org.ntnu.masteroppg.SearchResult;
import org.ntnu.masteroppg.SearchResults;
import org.ntnu.masteroppg.medline.MedlineIndexSearch;
import org.ntnu.masteroppg.medline.MedlineIndexSearch.SearchAlgorithm;

public class TrecEval {
	
	private static final int NUMBER_OF_TOPICS = 50;
	private static final String DEFAULT_TAG = "tag1";
	private String tag = DEFAULT_TAG;
		
	/*
	 * Example result: 
	 *   1 Q0 12474524 1  5567      tag1
	 * 
	 * From: http://ir.ohsu.edu/genomics/2003protocol.html
	 *   where: 
    * The first column is the topic number (1-50).
    * The second column is the query number within that topic. This is currently unused and must always be Q0.
    * The third column is the official PubMedID of the retrieved document.
    * The fourth column is the rank the document is retrieved
    * The fifth column shows the score (integer or floating point) that generated the ranking. This score MUST be in descending (non-increasing) order. The trec_eval program ranks documents based on the scores, not the ranks in column 4. If a submitter wants the exact ranking submitted to be evaluated, then the SCORES must reflect that ranking.
    * The sixth column is called the "run tag" and must be a unique identifier across all runs submitted to TREC. Thus, each run tag should have a part that identifies the group and a part that distinguishes runs from that group. Tags are restricted to 12 or fewer letters and numbers, and *NO* punctuation, to facilitate labeling graphs and such with the tags.
 */
	private static final String RES_TPL = "%d\t%s\t%s  \t%d\t%f\t%s";
	private static final String FILE_TPL = "%s_results_%s";
	
	private static final String DUMMY_SUBQUERY_NR = "Q0";
	private Logger log = Logger.getLogger(this.getClass());
	private MedlineIndexSearch search;
	
	public TrecEval() {
		
	}

	public void run() throws ParseException, IOException {
		
		SearchAlgorithm toTest[] = { 
				SearchAlgorithm.SENTENCE, 
				//SearchAlgorithm.LUCENE_DEFAULT,
				//SearchAlgorithm.BM25,
				SearchAlgorithm.BM25SENTENCE
			};
		
		//runQuerySet(toTest, TITLE_NEED);
		//runQuerySet(toTest, CUSTOM_QUERIES);
		runQuerySet(toTest, CUSTOM_QUERIES2);
		//runQuerySet(toTest, TOPIC_QUERIES);
		
		
		
	}

	private void runQuerySet(SearchAlgorithm[] toTest, String[] querySet) throws ParseException, IOException {
		if (querySet.length != NUMBER_OF_TOPICS) {
			throw new AssertionError("query set should have size " 
					+ NUMBER_OF_TOPICS + ". Is " + querySet.length);
		}
		long time = System.currentTimeMillis() / 1000L;
		for (SearchAlgorithm alg : toTest) {
			
			try {
				search = new MedlineIndexSearch(alg);
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
			}
			
			int i = 0;
			SearchResults[] allResults = new SearchResults[querySet.length];
			for (String query : querySet) {
				log.info("Searching for " + query);
				allResults[i++] = search.search(query, 
						MedlineIndexSearch.SearchFields.ABSTRACT);	
			}
			
			//prettyPrint(allResults);
			storeResults(allResults, alg, time);
		}
	}
	
	private void storeResults(SearchResults[] results, SearchAlgorithm algorithm, long testStart) throws IOException {
		String fileName = String.format(FILE_TPL, 
				algorithm.name(),
				testStart // unixtime
		);
		String path = Config.getStr(Config.EVAL_RES_PATH);
		path += "/" + fileName;
		BufferedWriter writer = new BufferedWriter(
				new FileWriter(path));
		
		int topic = 1;
		for (SearchResults res : results) {
			Iterator<SearchResult> itr = res.iterator();
			while (itr.hasNext()) {
				writer.write(formatRes(topic, itr.next()) + "\n");
			}
			topic++;
		}
		writer.close();
	}

	private void prettyPrint(SearchResults[] allResults) {
		int topic = 1;
		for (SearchResults res : allResults) {
			System.out.println("Topic " + topic 
					+ " query: " + TOPIC_QUERIES[topic -1]);
			Iterator<SearchResult> itr = res.iterator();

			while (itr.hasNext()) {
				SearchResult r = itr.next();
				if (r == null)
					throw new AssertionError();
				System.out.println(formatRes(topic, r));
			}
			topic++;
			System.out.println();
		}

	}

	private String formatRes(int queryid, SearchResult r) {
		return String.format(RES_TPL, 
				queryid,
				DUMMY_SUBQUERY_NR,
				r.getDocID(), 
				r.getRank(),
				r.getScore(),
				tag);
	}
	
	private static final String[] CUSTOM_QUERIES2 = {
		"iron AND (Ferroportin1 OR (Ferroportin AND 1) OR SLC40A1 OR FPN1 OR HFE4 OR IREG1 OR (Iron AND regulated AND gene AND 1) OR MTP1 OR SLC11A3)",                                         
		"(transgenic OR (copy AND gene)) AND (mice OR mouse OR murine)",                                 
		"kidney AND (mice OR mouse OR murine) AND (develop OR developing OR development)",               
		"kidney AND ((gene OR genes) OR (expression AND (profile OR profile))) AND (mice OR mouse OR murine)",
		 "(isolate OR isolating OR fractionation OR purify) AND cell",
		"FancD2 OR (Fanconi AND anemia) OR (group D2) OR (type AND 4 AND fanconi AND pancytopenia)",     
		"(oxidative OR oxidation) AND DNA AND (repair OR effect)",                                       
		"((oxidative AND (disease OR diseases OR carcinogenesis)) OR (DNA AND repair)) AND (cancer OR cancers OR carcinoma)",
		"(muty OR hmyh) AND -myoglobin",                                                                 
		"(NEIL1 OR (nei AND endonuclease)) AND (DNA OR repair)",                                         
		"hairless mice carcinogenesis skin OR UV",                                                       
		"(gene OR genes) AND smad4",                                                                                         
		"(TGFB OR (transforming growth factor beta)) AND (homeostasis OR angiogenesis)",
		"(TGFB OR (transforming growth factor beta)) AND ((head and neck squamous cell) OR HNSCC) AND (cancer OR cancers OR carcinoma)",                                                                  
		"(ATPase OR ATPases) AND (apoptosis OR (cell death))",                                           
		"(AAA OR (ATPases associated activities)) AND (protein OR proteins OR lipids OR DNA)",           
		"(DO1 OR (p53 AND (antibody OR anti))) AND binding",  
		"(Gis4 OR YML006C)",                                                   
		"(GAL1 OR SUC1) AND (repressors OR reprosessor OR activators OR activator)",            
		"(covalent OR attachment OR covalence OR substrate) AND (ubiquitin OR ubiquitously OR ubiquitylation OR ubiquitination)",
		"(p63 OR TP63) OR (TP73 OR p73) DNA",
		"p53 DNA (break OR damage)",
		"Saccharomyces OR cerevisiae (protein OR proteins) (ubiquitin OR proteolytic OR pathway)",
		"(mice OR mouse OR murine) AND (peptidoglycan OR PGRP or PGRPs)",
		"scleroderma OR (autoimmune disease (genes OR gene))",
		"(BUB2 OR BFA1) AND (cytokinesis OR yeast)",
		"(autophagy OR (gene autophagic)) AND apoptosis ",
		"(autophagy OR (gene autophagic)) AND apoptosis AND (proteases OR morphological)",
		"(gyrA OR (DNA gyrase)) AND (mutation OR mutations OR alteration)",
		"Nkx OR Sax",
		 "TOR OR (Target AND Of AND Rapamycin) OR FRAP1 OR (FK506 AND associated AND protein)",
		"Xenograft AND (tumorogenesis OR cancer OR cancers OR carcinoma)",
		"(Histoplasmosis OR (blood borne hogen)) AND (mice OR mouse OR murine)",
		"Cryptococcus AND (gene OR genes OR genome)",
		"WD40 OR (protein AND repeats)",
		 "(RAB3A OR (RAS oncogene family)) AND ((synaptic plasticity) OR synapse)",
		"PAM OR (peptide AND amidating AND enzyme) OR (peptidylglycine AND amidating)",
		"(genetic AND loci) OR Stroke OR E4",
		"Hypertension ((risk OR danger) AND stroke)",
		 "(antigen OR antigens) AND (epithelial OR epithelium)",
		"(mutation OR mutations OR altered) AND ((Cystic AND Fibrosis) OR CF OR mucovoidosis OR muscoviscidosis)",
		"(chromosome AND translocations) OR (chromosomal (rearrangement OR rearrangements))",
		"(sleeping AND beauty) OR SB",
		"(nerve AND growth AND factor) OR NFG",
		"(MWH1 OR (mental health wellness)) OR (mental disorder gene)",
		"RSK2 OR (ribosomal protein kinase)",
		"BCL OR BCL2 OR (BCL AND 2)",
		"(UNC OR BGS OR homologues) AND (gene OR genes)",
		 "(Glyphosate OR glycine) AND (tolerance OR tolerant OR immune)",
		"low temperature ((E AND coli) OR escherichia)"
	};
	
	
	private static final String[] CUSTOM_QUERIES = {
		"Ferroportin1 OR SLC40A1 OR FPN1 OR HFE4 OR IREG1", 
		"transgenic mice",                                  
		"murine OR mouse kidney generate",
		"kidney gene profiles mice OR mouse",
		"isolate OR isolating cell nuclei",
		"FancD2 OR Fanconi anemia",
		"oxidative DNA repair",
		"DNA repair cancer",
		"muty OR hmyh",
		"NEIL1 DNA",
		"hairless mice carcinogenesis skin OR UV",
		"smad4",
		"TGFB angiogenesis OR skin AND (ostasis OR development)",
		"TGFB OR HNSCC AND (cancer OR cancers)",
		"ATPase OR apoptosis",
		"AAA (protein OR proteins OR lipids OR DNA)",
		"DO1 OR p53 OR (monoclonal antibody)",
		"Gis4 cycle OR metabolism",
		"(GAL1 OR SUC1) AND (repressors OR reprosessor OR activators OR activator)",
		"(modified OR modify) covalent (ubiquitin OR protein)",
		"p63 OR p73 damage",
		"p53 DNA (break OR damage)",
		"Saccharomyces OR cerevisiae (protein OR proteins) (ubiquitin OR proteolytic OR pathway)",
		"mouse (peptidoglycan OR PGRP)",
		"scleroderma",
		"(BUB2 OR BFA1) AND (cytokinesis OR yeast)",
		"(role OR interconnection) autophagy apoptosis",
		"autophagy aptosis OR (cell AND death)",
		"gyrA (mutation OR mutations) (sequence OR sequences OR genotypes OR genotype)",
		"Nkx (regulatory OR regulated)",
		"TOR neurofibromatosis (signaling OR signal)",
		"Xenograft (tumorogenesis OR cancer OR cancers)",
		"(histoplasmosis OR (blood borne hogen)) AND (mice OR mouse)",
		"cyptococcus (annotation OR annotated)",
		"WD40",
		"RAB3A OR (synaptic plasticity)",
		"PAM",
		"stoke risk (E4 OR (factor AND V))",
		"hypertension (risk AND stroke)",
		"(antigen OR antigens) AND (eppithelial AND (cell OR cells))",
		"(mutation OR mutations OR altered) Cystic Fibrosis",
		"chromosome translocations (gene OR genes)",
		"sleeping beauty",
		"nerve growth (proteins OR protein)",
		"(MWH1 OR (mental health wellness))",
		"RSK2",
		"BCL OR BCL2 OR (BCL AND 2) (antagonists OR antagonist OR inhabitors OR inhabitor)",
		"UNC (gene OR genes)",
		"glyphosate tolerance (gene OR genes)",
		"low temperature ((E AND coli) OR escherichia)"
	};

	private static final String[] TOPIC_QUERIES = {
		"Ferroportin-1 iron transporter humans ",
		"transgenic mice ",
		"kidney ",
		"mouse genes kidney ",
		"nuclei ",
		"FancD2 ",
		"DNA repair pathways oxidative stress ",
		"DNA repair oxidative diseases skin carcinogenesis UV carcinogenesis ",
		"mutY humans ",
		"NEIL1 repair ",
		"Find articles hairless mice ",
		"signal transducing molecule Smad4 ",
		"TGFB skin ",
		"TGFB expression HNSCC cancers ",
		"ATPases ",
		"AAA proteins lipids ",
		"anti-p53 monoclonal antibody DO1 ",
		"Gis4 ",
		"upstream promoter regions GAL1 SUC1 co activators SNF1 ",
		"ubiquitin ubiquitin like proteins ",
		"p63 p73 ",
		"p53 ",
		"Saccharomyces cerevisiae ",
		"mouse peptidoglycan recognition proteins PGRP ",
		"Identify studies scans and microarray analysis scleroderma ",
		"BUB2 BFA1 yeast ",
		"apoptosis ",
		"morphological changes apoptosis autophagy processes ",
		"E. coli gyrA mutations ",
		"Nkx gene family members ",
		"TOR signaling ",
		"xenograft models human cancers ",
		"mutant mouse strains Histoplasma capsulatum ",
		"Cryptococcus. ",
		"WD40 repeats ",
		"RAB3A ",
		"PAM ",
		"E4 or factor V mutations ",
		"Identify genes ",
		"antigens lung epithelial cells antibodies ",
		"Cystic Fibrosis conductance ",
		"chromosomal rearrangements ",
		"Sleeping Beauty transposons ",
		"nerve growth factor pathway proteins ",
		"Mental Health Wellness 1 MWH1 mental ",
		"human biological processes ",
		"human gene BCL-2 ",
		"human UNC gene family ",
		"public domain DNA segment ",
		"Escherichia coli bacteria ",
		};
	
	private static final String[] TITLE_NEED = {                
        "Ferroportin-1 in humans",                          
        "Generating transgenic mice",
        "Time course for gene expression in mouse kidney",
        "Gene expression profiles for kidney in mice",
        "Protocols for isolating cell nuclei",
        "FancD2",
        "DNA repair and oxidative stress",
        "Correlation between DNA repair pathways and skin cancer",
        "mutY",
        "NEIL1",
        "Carcinogenesis and hairless mice",
        "Genes regulated by Smad4",
        "Role of TGFB in angiogenesis in skin",
        "Expression or Regulation of TGFB in HNSCC cancers",
        "ATPase and apoptosis",
        "AAA proteins",
        "DO1 antibody",
        "Gis4",
        "Comparison of Promoters of GAL1 and SUC1",
        "Substrate modification by ubiquitin",
        "Role of p63 and p73 in relation to DNA damage",
        "Relative response of p53 family members to agents causing single stranded versus double stranded DNA breaks",
        "Saccharomyces cerevisiae proteins involved in ubiquitin system",
        "Mouse peptidoglycan recognition proteins (PGRP)",
        "Cause of scleroderma",
        "Function of BUB2/BFA1 in the process of cytokinesis",
        "Role of autophagy in apoptosis",
        "Proteases that function in both apoptosis and autophagy cell death",
        "Phenotypes of gyrA mutations",
        "Regulatory targets of the Nkx gene family members",
        "TOR signaling in neurofibromatosis",
        "Xenograft animal models of tumorogenesis",
        "Mice, mutant strains, and Histoplasmosis",
        "Gene products of Cryptococcus important to fungal survival",
        "WD40 repeat containing proteins",
        "RAB3A",
        "PAM",
        "Risk factors for stroke",
        "Hypertension",
        "Antigens expressed by lung epithelial cells",
        "Mutations in the Cystic Fibrosis conductance regulator gene",
        "Genes altered by chromosome translocations",
        "Sleeping Beauty",
        "Proteins involved in the nerve growth factor pathway",
        "Mental Health Wellness-1",
        "RSK2",
        "Human gene BCL-2 antagonists and inhibitors",
        "Human homologues of C. elegans UNC genes",
        "Glyphosate tolerance gene sequence",
        "Low temperature protein expression in E. coli",
};

}
