package org.ntnu.masteroppg.index;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.zip.CRC32;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Fieldable;
import org.ntnu.masteroppg.Config;

import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.util.AbstractExternalizable;

/**
 * Adds token to sentence index, based on sentence weight.
 * 
 */
public class SentenceWeighter {
	
	Map<Long, ArrayList<WeightingSentence>> cache 
		= new ConcurrentHashMap<Long, ArrayList<WeightingSentence>>();
	
	private Chunker chunker;
	private SentenceIndex index;
	private Logger log = Logger.getLogger(this.getClass());

	private SentenceData sentenceData;
	
	public SentenceWeighter(SentenceIndex index, SentenceData sentenceData) {
		loadChunker();
		this.index = index;
		this.sentenceData = sentenceData;
	}

	private void weight(Fieldable field) {
		//log.debug("Weighting: " + field.stringValue());
		ArrayList<WeightingSentence> weights = weightSentences(field);
		cache.put(fieldHash(field), weights);
	}

	private boolean inCache(long i) {
		if (cache.containsKey(i)) {
			//log.debug("token " + i + " is in cache");
			return true;
		}
		return false;
	}
	
	

	private ArrayList<WeightingSentence> weightSentences(Fieldable field) {
		final ArrayList<WeightingSentence> sent = new ArrayList<WeightingSentence>();
		
		SentenceIterator sitr = new LangPipeSentenceIterator(field.stringValue()) {
			public void handleNext(String sentence, int start, int end, int tokens) {
				final int weight = weightString(sentence);
				
				
				final float normalized = normalize(weight, tokens);
			//	log.debug("weight " + weight + " (" + normalized + ") on sentence: " + sentence);
				//System.err.println("normalized: " + normalized + " tokens: " + tokens  + " weight: " + weight);
				sent.add(new WeightingSentence(end, normalized));
			}
			
			final float k_1 = 1.2f; // BM25 constants
			final float b = 0.75f;
			final float avgSentLen = sentenceData.getSentenceAvg();

			private float normalize(float weight, float sentLen) {
				
				final float K = k_1 * (1f - b + (b * (sentLen / avgSentLen)));
				final float normalized = (((k_1 + 1) * (weight + 1)) / (K + (weight + 1)));
				
				
				
				/*if (normalized < 1)
					throw new AssertionError(normalized);*/
				if (normalized <= 0 || normalized >= 2) {
					log.debug("normalized: " + normalized + " weight: " + weight + " senlen:" + sentLen);
				}
				
				
				return normalized;
				
			}
		};
		sitr.run();

		return sent;
		
}
	
	private int weightString(String str) {
		Chunking c = chunker.chunk(str);
		//return c.chunkSet().size();
		
		//System.out.println("input=" + s +
			//	"\tchunking=" + c);


		Set<Chunk> set = c.chunkSet();
		return set.size();
		/*Iterator<Chunk> it = set.iterator();
		int weight = 0;
		while (it.hasNext()) {
			Chunk chunk = it.next();
			//if (chunk.type().equals("other_name"))
			//	continue;
	           //Chunk chunk = (Chunk) it.next();
	           //int start = chunk.start();
	           //int end = chunk.end();
	           //String text = s.substring(start,end);
	           //System.out.println("     chunk=" + chunk + "  text=" + text);
	           //System.out.println("Matched type: " + chunk.type());
	           //s.addMatch(chunk.type());
	         weight++;
		}
		return weight;*/
	}
	private void loadChunker() {
		File genia = new File(Config.getStr(Config.GENIA_CHUCKER_PATH));
		//File genia = new File(Config.getStr(Config.GENETAG_CHUNKER_PATH));
		try {
			chunker = (Chunker) AbstractExternalizable.readObject(genia);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	
	private WeightingSentence popSentence(long fieldHash) {
		//log.debug("popping sentence");
		WeightingSentence s = cache.get(fieldHash).get(0);
		cache.get(fieldHash).remove(0);
		return s;
	}
	private static class IndexState {
		public static Fieldable prevField = null;
		public static int prevTokenPos = -1;
		public static int prevTokenEnd = -1;
		public static WeightingSentence currSentence = null;
		public static boolean ignoreDocument = false;
		
		public static void clear() {
			currSentence = null;
			prevTokenEnd = -1;
			prevTokenPos = -1;
			prevField = null;
			ignoreDocument = false;
		}
	}
	
	private long fieldHash(Fieldable field) {
		String value = field.stringValue() + " -- " + field.name();
		CRC32 crc32 = new CRC32();
		crc32.update(value.getBytes());
		
		final long hash = crc32.getValue();
		return hash;
	}

	public void tokenToIndex(Fieldable field, int tokenPos, int tokenEnd) {
		//log.debug("token: " + tokenEnd);
		// field == null means there are no more tokens left.
		if (field == null) {
			// if IndexState holds a sentence, asume prev token
			// was the last token in that sentence.
			
			
			if (IndexState.ignoreDocument) {
				//log.debug("ignoring token document");
				IndexState.clear();
				return;
			}
			
			if (IndexState.prevField == null) { // exact end match on last sentence
				//log.debug("exact end match on prev sentence");
				IndexState.clear();
				return;
			}	
			
		
			if (cache.get(fieldHash(IndexState.prevField)).size() != 0) {
				log.fatal("Should be at last sentence, but cache is not empty.");
				log.fatal("In cache: " + cache.get(fieldHash(IndexState.prevField)).size());
				log.fatal("Document: " + IndexState.prevField.stringValue());
				dumpSentences(fieldHash(IndexState.prevField));
				return;
			}
			
			cache.remove(fieldHash(IndexState.prevField)); // cleare from cache
			
			log.debug("tokenEnd: " + IndexState.prevTokenEnd + " (last)");

			index.addSentence(
					IndexState.prevField.getSentenceIndexPtr(),
					IndexState.prevTokenPos, 
					IndexState.currSentence.weight());
			IndexState.clear();
			return;
			
		}
		if (IndexState.ignoreDocument) {
		//	log.debug("ignoring document");
			return;
		}
		
		
		long hash = fieldHash(field);
		if (!inCache(hash))
			weight(field);

		
		if (IndexState.currSentence == null) {
			IndexState.currSentence = popSentence(hash);	
		}		
		
		if (IndexState.currSentence.end() == tokenEnd) {
			// matches an end of a sentence
			log.debug("tokenEnd: " + tokenEnd + " (exact)");
			index.addSentence(
					field.getSentenceIndexPtr(), 
					tokenPos,
					IndexState.currSentence.weight());
			IndexState.clear();

		}
		else if (IndexState.currSentence.end() < tokenEnd) {
			// passed a sentence. 
			// asume previous token was last token in sentence.
			if (IndexState.prevTokenEnd == -1) {
				//throw new AssertionError();
				IndexState.ignoreDocument = true;
				log.fatal("no token found in sentence, sentence mismatch? Ignoring document.");
				return;
			}
			log.debug("tokenEnd: " + IndexState.prevTokenEnd + " (prev)");
			index.addSentence(
					IndexState.prevField.getSentenceIndexPtr(),
					IndexState.prevTokenPos, 
					IndexState.currSentence.weight());
			IndexState.clear();
			IndexState.currSentence = popSentence(hash);
		}
		else {
			// not at an end of a sentence, upd prev
			//log.debug("not at the end of a sentence, upd prev");
			IndexState.prevField = field;
			IndexState.prevTokenEnd = tokenEnd;
			IndexState.prevTokenPos = tokenPos;
		}
	}

	private void dumpSentences(long hash) {
		ArrayList<WeightingSentence> s = cache.get(hash);
		for (int i = 0; i < s.size(); i++) {
			log.debug("#" + i + " sentence: " + s.get(i));
		}
		
		
	}
}

/**
 * Stores info needed before sentence is added to index
 */
class WeightingSentence {
	private int end; /* char-pos where sentence ends in field string */
	private float weight;
	public WeightingSentence(int end, float weigth) {
		this.end = end;
		this.weight = weigth;
	}
	public int end() {
		return end;
	}
	public float weight() {
		return weight;
	}
	public String toString() {
		return "WeightingSentence - end: " + end + " weight: " + weight;
	}
	
}