package org.ntnu.masteroppg.luceneExt;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.ntnu.masteroppg.biotracer.CustomPhraseQuery;
import org.ntnu.masteroppg.biotracer.CustomTermQuery;
import org.ntnu.masteroppg.index.SentenceIndex;
import org.ntnu.masteroppg.medline.MedlineIndexSearch.SearchAlgorithm;

public class QueryRewriter {
	
	private Analyzer analyzer;
	private SentenceIndex sentenceIndex;

	public QueryRewriter(Analyzer a, SentenceIndex sentenceIndex) {
		this.analyzer = a;
		this.sentenceIndex = sentenceIndex;
	}
	
	public Query parse(String query, String field, SearchAlgorithm algorithm) throws ParseException {
		LocalParser parser = new LocalParser(
				field, this.analyzer, this.sentenceIndex, algorithm);
		return parser.parse(query);
		
	}
}


class LocalParser extends QueryParser {
	private final Logger log = Logger.getLogger(this.getClass());
	private SentenceIndex sentenceIndex;
	private SearchAlgorithm algorithm; 

	public LocalParser(String f, Analyzer a, SentenceIndex sentenceIndex, SearchAlgorithm algorithm) {
		super(f, a);
		this.algorithm = algorithm;
		this.sentenceIndex = sentenceIndex;
	}
	  public Query parse(String query) throws ParseException {
		  if (sentenceIndex == null)
			  throw new AssertionError();
		  log.debug("parse: " + query);
		  
		  
		  Query q = super.parse(query);
		  
		  
		  return replaceQuery(q);
	  }
	private Query replaceQuery(Query q) throws AssertionError, ParseException {
		if (q instanceof TermQuery) {
			  TermQuery tq = (TermQuery) q;
			  return createTermQuery(tq);
		  }
			
		  else if (q instanceof BooleanQuery) {
			  replaceBooleanTermQuery((BooleanQuery) q);
			  return q;
		  }
		  else if (q instanceof PhraseQuery) {
			  throw new ParseException("phrase query not supported");
			  //return rewritePhraseQuery((PhraseQuery) q);  
		  }
			
		  else if (q instanceof PrefixQuery)
			  throw new ParseException("prefix query not supported");
		  else {
			  throw new ParseException("unsupported query type");
		  }
	}
	private Query rewritePhraseQuery(PhraseQuery oldQuery) {
		Term terms[] = oldQuery.getTerms();
		int pos[] = oldQuery.getPositions();
		switch (algorithm) {
		case SENTENCE:
			PhraseQuery mpq = new SentencePhraseQuery(sentenceIndex);
			for (int j = 0; j < terms.length; j++)
				mpq.add(terms[j], pos[j]);
			return mpq;

		case BM25:
		case BM25SENTENCE:
			CustomPhraseQuery cpq = new CustomPhraseQuery();
			for (int j = 0; j < terms.length; j++)
				cpq.add(terms[j], pos[j]);
			return cpq;
			
		default:
			throw new AssertionError("algorithm not implemented");
		}		
	}
	private Query createTermQuery(TermQuery tq) throws AssertionError {
		switch (algorithm) {
		  case SENTENCE:
			  return new SentenceTermQuery(tq.getTerm(), sentenceIndex);
		  case BM25:
			  return new CustomTermQuery(tq.getTerm());
		  case BM25SENTENCE:
			  return new CustomTermQuery(tq.getTerm(), sentenceIndex);
		  default:
			  throw new AssertionError("Unsupported algorithm");
		  }
	}
	  
	  private void replaceBooleanTermQuery(BooleanQuery boolQuery) throws ParseException {
		  
		  for (BooleanClause clause : boolQuery.getClauses()) {
			  Query q = clause.getQuery();
			  Query newQ = replaceQuery(q);
			  clause.setQuery(newQ);
			  
		  }
	  }
}
