package org.ntnu.masteroppg.medline;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.ntnu.masteroppg.Config;
import org.ntnu.masteroppg.SearchResults;
import org.ntnu.masteroppg.biotracer.CustomSimilarity;
import org.ntnu.masteroppg.index.SentenceIndex;
import org.ntnu.masteroppg.luceneExt.QueryRewriter;
import org.ntnu.masteroppg.luceneExt.SentenceSimilarity;

public class MedlineIndexSearch {
	
	public enum SearchFields { TITLE, ABSTRACT };
	public enum SearchAlgorithm { LUCENE_DEFAULT, SENTENCE, BM25, BM25SENTENCE };

	private IndexSearcher searcher;
	private static Logger log = null;
	//private MultiFieldQueryParser bothQueryParser;
	//private MedlineQueryParser abstractQueryParser;
	//private QueryParser titleQueryParser;
	private SentenceIndex sentenceIndex;
	private Analyzer analyzer = new StandardAnalyzer();
	
	private QueryRewriter custParser;
	private SearchAlgorithm searchAlgorithm;

	public MedlineIndexSearch(SearchAlgorithm alg) throws IOException {
		String indexPath = null;
		String sentenceIndexPath = null;;
		this.searchAlgorithm = alg;
		
		switch (alg) {
		case BM25:
		case BM25SENTENCE:
			indexPath = Config.getStr(Config.BM25_INDEX_PATH);
			sentenceIndexPath = Config.getStr(Config.BM25_SENTENCE_INDEX_PATH);
			break;
		case LUCENE_DEFAULT:
		case SENTENCE:
			indexPath = Config.getStr(Config.DEFAULT_INDEX_PATH);
			sentenceIndexPath = Config.getStr(Config.DEFAULT_SENTENCE_INDEX_PATH);
			break;
		default:
			throw new AssertionError("unsupported algorithm");
		}
		Directory dir = FSDirectory.getDirectory(indexPath);
		searcher = new IndexSearcher(dir);
		
		
		try {
			sentenceIndex = SentenceIndex.load(sentenceIndexPath);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		 custParser = new QueryRewriter(analyzer, sentenceIndex);
		
		if (log == null)
			log = Logger.getLogger(this.getClass());

	}

	public SearchResults search(String querystr, SearchFields field) throws ParseException, IOException {
		Query query = null;
		
		if (sentenceIndex == null)
			throw new AssertionError();
		
		
		switch (field) {
		case ABSTRACT:
			query = parseQuery(searchAlgorithm, 
					Config.getStr(Config.DOC_FIELD_ABSTRACT), querystr);
			break;
		default:
			throw new AssertionError("Unsupported field");
		}		
		
		log.info("Searching " + field.toString() 
				+ " for " + querystr + " with " + searchAlgorithm.toString());
		
		SearchResults result = new SearchResults(
				searcher, searcher.search(query, null, Config.getInt(Config.NUM_RESULTS)));
		return result;
		
	}
	
	private Query parseQuery(SearchAlgorithm algorithm, String field, String query) throws ParseException {
		
		
		switch (algorithm) {
		case SENTENCE:
			//searcher.setSimilarity(new SentenceSimilarity());
			return custParser.parse(query, field, algorithm);
			
		case LUCENE_DEFAULT:
			searcher.setSimilarity(Similarity.getDefault());
			return (new QueryParser(field, analyzer)).parse(query);
			
		case BM25:
			CustomSimilarity sim = new CustomSimilarity();
			sim.extractQueryTermFreq(new String[] {
					Config.getStr(Config.DOC_FIELD_ABSTRACT) },
					query
			);
			
			searcher.setSimilarity(sim);
			return custParser.parse(query, field, algorithm);
			
		case BM25SENTENCE:
			CustomSimilarity sim1 = new CustomSimilarity(
					CustomSimilarity.Model.BM25_SENTENCE
			);
			sim1.extractQueryTermFreq(new String[] {
					Config.getStr(Config.DOC_FIELD_ABSTRACT) },
					query
			);
			searcher.setSimilarity(sim1);
			return custParser.parse(query, field, algorithm);
			
			
		default:
			throw new AssertionError("Unsupported search algorithm");
		}
	}

	public void finalize() {
		try {
			searcher.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	

}
