package org.ntnu.masteroppg.medline;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.ntnu.masteroppg.Config;
import org.ntnu.masteroppg.biotracer.CustomSimilarity;
import org.ntnu.masteroppg.index.SentenceData;
import org.ntnu.masteroppg.index.SentenceIndex;
import org.ntnu.masteroppg.index.SentenceWeighter;
import org.ntnu.masteroppg.luceneExt.SentenceAnalyzer;
import org.ntnu.masteroppg.medline.MedlineIndexSearch.SearchAlgorithm;

public class MedlineIndexWriter {
	
	private IndexWriter iwriter;
	private int added = 0;
	private Logger log;
	private SearchAlgorithm searchAlgorithm;
	private static SentenceIndex sentenceIndex;
	private static SentenceWeighter sentenceWeighter;
	
	public MedlineIndexWriter(SentenceData sentenceData, SearchAlgorithm algo) throws CorruptIndexException, LockObtainFailedException, IOException {
		sentenceIndex = null;
		sentenceWeighter = null;
		
		log = Logger.getLogger(this.getClass());
		this.searchAlgorithm = algo;
		
	/*	Directory defaultDir = FSDirectory.getDirectory(Config.getStr(Config.DEFAULT_INDEX_PATH));
		Directory bm25Dir = FSDirectory.getDirectory(Config.getStr(Config.BM25_INDEX_PATH));*/
		Analyzer a = new SentenceAnalyzer();
		
		switch (algo) {
		case BM25:
		case BM25SENTENCE:
			Directory b25dir = FSDirectory.getDirectory(Config.getStr(Config.BM25_INDEX_PATH));
			log.debug("To dir: " + b25dir);
			iwriter = new IndexWriter(b25dir, a, true, IndexWriter.MaxFieldLength.LIMITED);
			iwriter.setSimilarity(new CustomSimilarity(CustomSimilarity.Model.BM25));
			break;
		case LUCENE_DEFAULT:
		case SENTENCE:
			Directory ddir = FSDirectory.getDirectory(Config.getStr(Config.DEFAULT_INDEX_PATH));
			log.debug("To dir: " + ddir);
			iwriter = new IndexWriter(ddir, a, true, IndexWriter.MaxFieldLength.LIMITED);
			break;
		default:
			throw new AssertionError("unsupported algorithm");
		}
		
		
		sentenceIndex = new SentenceIndex();
		sentenceWeighter = new SentenceWeighter(sentenceIndex, sentenceData);
	}
	public void addDocument(String titleText, String abstrText, String pmid, Integer docLength) throws CorruptIndexException, IOException {
		Document doc = new Document();

		addAbstrField(abstrText, doc);
		
		
		
		Field titleField = new Field(
				Config.getStr(Config.DOC_FIELD_TITLE), 
				titleText, 
				Field.Store.YES, Field.Index.NO
		);
		Field pmidField = new Field(
				Config.getStr(Config.DOC_FIELD_PMID),
				pmid,
				Field.Store.YES, Field.Index.NO
		);
		Field docLenField = new Field(
				Config.getStr(Config.DOC_FIELD_DOCLEN),
				docLength.toString(),
				Field.Store.YES, Field.Index.NO
		);


		doc.add(titleField);
		doc.add(pmidField);
		doc.add(docLenField);
		
		iwriter.addDocument(doc);
		//if ((added++ % 500) == 0)
		log.info("Added " + ++added + ", in index: " + iwriter.numDocs());
		//log.debug("Text: " + abstrText);

	}

	private void addAbstrField(String abstrText, Document doc) throws AssertionError {
		Field abstrField = new Field(
				Config.getStr(Config.DOC_FIELD_ABSTRACT), 
				abstrText,
				Field.Store.YES, Field.Index.ANALYZED
		);
		//if (searchAlgorithm == SearchAlgorithm.BM25 || searchAlgorithm == SearchAlgorithm.BM25SENTENCE) {
			int indexid = sentenceIndex.addDoc();
			abstrField.setSentenceIndexPtr(indexid);
		//}
		doc.add(abstrField);


	}
	
	public void close() {
		try {
			log.info("Committing index, and closing.");
			iwriter.commit();
			iwriter.close();
			
			String sentenceIndexPath = null;
			switch (searchAlgorithm) {
			case BM25:
			case BM25SENTENCE:
				sentenceIndexPath = Config.getStr(Config.BM25_SENTENCE_INDEX_PATH);
				break;
			case LUCENE_DEFAULT:
			case SENTENCE:
				sentenceIndexPath = Config.getStr(Config.DEFAULT_SENTENCE_INDEX_PATH);
				break;
			default:
				throw new AssertionError("unsupported algortihm");
			}
			
			sentenceIndex.store(sentenceIndexPath);
			sentenceIndex.printDump();
			
		} catch (CorruptIndexException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void finalize() {
		close();
		try {
			super.finalize();
		} catch (Throwable e) {
			Logger.getLogger(this.getClass()).error(
					"Finalize error '" + e.getMessage() + "' ignored."
			);
		}
	}

	

	public static void addSentenceIndexToken(Fieldable field, int position, int end) {
		sentenceWeighter.tokenToIndex(field, position, end);
	}

	
		
	

}